﻿using System;
using System.Threading;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Threading;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Delegate
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Delegate

        /// <summary>
        /// 단순 대리자
        /// </summary>
        private delegate void SimpleDelegate();

        /// <summary>
        /// 단일 인자 대리자
        /// </summary>
        /// <param name="argument">인자</param>
        private delegate void SingleArgumentDelegate(string argument);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 시계 표시하기 스토리보드
        /// </summary>
        private Storyboard showClockStoryboard;

        /// <summary>
        /// 시계 숨기기 스토리보드
        /// </summary>
        private Storyboard hideClockStoryboard;

        /// <summary>
        /// 날씨 표시하기 스토리 보드
        /// </summary>
        private Storyboard showWeatherStoryboard;

        /// <summary>
        /// 날씨 숨기기 스토리 보드
        /// </summary>
        private Storyboard hideWeatherStoryboard;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            Loaded                 += Window_Loaded;
            this.fetchButton.Click += fetchButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.showClockStoryboard   = (Storyboard)Resources["ShowClockStoryboardKey"  ];
            this.hideClockStoryboard   = (Storyboard)Resources["HideClockStoryboardKey"  ];
            this.showWeatherStoryboard = (Storyboard)Resources["ShowWeatherStoryboardKey"];
            this.hideWeatherStoryboard = (Storyboard)Resources["HideWeatherStoryboardKey"];
        }

        #endregion
        #region Fetch Forecast 버튼 클릭시 처리하기 - fetchButton_Click(sender, e)

        /// <summary>
        /// Fetch Forecast 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void fetchButton_Click(object sender, RoutedEventArgs e)
        {
            this.fetchButton.IsEnabled = false;

            this.fetchButton.Content = "Contacting Server";

            this.weatherTextBlock.Text = string.Empty;

            this.hideWeatherStoryboard.Begin(this);

            SimpleDelegate simpleDelegate = new SimpleDelegate(FetchWeatherFromServer);

            simpleDelegate.BeginInvoke(null, null);
        }

        #endregion
        #region 시계 숨기기 스토리보드 완료시 처리하기 - hideClockStoryboard_Completed(sender, e)

        /// <summary>
        /// 시계 숨기기 스토리보드 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void hideClockStoryboard_Completed(object sender, EventArgs e)
        {
            this.showWeatherStoryboard.Begin(this);
        }

        #endregion
        #region 날씨 숨기기 스토리보드 완료시 처리하기 - hideWeatherStoryboard_Completed(sender, e)

        /// <summary>
        /// 날씨 숨기기 스토리보드 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void hideWeatherStoryboard_Completed(object sender, EventArgs e)
        {
            this.showClockStoryboard.Begin(this, true);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region UI 업데이트하기 - UpdateUI(weather)

        /// <summary>
        /// UI 업데이트하기
        /// </summary>
        /// <param name="weather">날씨</param>
        private void UpdateUI(string weather)
        {
            if(weather == "sunny")
            {
                this.weatherImage.Source = (ImageSource)Resources["SunnyDrawingImageKey"];
            }
            else if(weather == "rainy")
            {
                this.weatherImage.Source = (ImageSource)Resources["RainingDrawingImageKey"];
            }

            this.showClockStoryboard.Stop(this);
            this.hideClockStoryboard.Begin(this);

            this.fetchButton.IsEnabled = true;
            this.fetchButton.Content   = "Fetch Forecast";

            this.weatherTextBlock.Text = weather;
        }

        #endregion
        #region 서버에서 날씨 가져오기 - FetchWeatherFromServer()

        /// <summary>
        /// 서버에서 날씨 가져오기
        /// </summary>
        private void FetchWeatherFromServer()
        {
            Thread.Sleep(4000);

            Random random = new Random();

            string weather;

            weather = random.Next(2) == 0 ? "rainy" : "sunny";

            this.rootGrid.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new SingleArgumentDelegate(UpdateUI), weather);
        }

        #endregion
    }
}